Method: Thor::Shell::Basic#error

Defined in:
lib/vendor/thor/lib/thor/shell/basic.rb

#error(statement) ⇒ Object

Called if something goes wrong during the execution. This is used by Thor internally and should not be used inside your scripts. If something went wrong, you can always raise an exception. If you raise a Thor::Error, it will be rescued and wrapped in the method below.



265
266
267
# File 'lib/vendor/thor/lib/thor/shell/basic.rb', line 265

def error(statement)
  stderr.puts statement
end