Class: EY::Serverside::CLI

Inherits:
Thor show all
Extended by:
CLIHelpers
Defined in:
lib/engineyard-serverside/cli.rb

Constant Summary

Constants inherited from Thor

Thor::HELP_MAPPINGS, Thor::THOR_RESERVED_WORDS, Thor::VERSION

Instance Attribute Summary

Attributes included from Thor::Base

#args, #options, #parent_options

Instance Method Summary collapse

Methods included from CLIHelpers

account_app_env_options, config_option, framework_env_option, instances_options, stack_option, verbose_option

Methods inherited from Thor

check_unknown_options!, check_unknown_options?, default_task, desc, help, #help, long_desc, map, method_option, method_options, printable_tasks, register, subcommand, subcommands, task_help

Methods included from Thor::Base

included, #initialize, register_klass_file, shell, shell=, subclass_files, subclasses

Instance Method Details

#deploy(default_task = :deploy) ⇒ Object



32
33
34
35
36
# File 'lib/engineyard-serverside/cli.rb', line 32

def deploy(default_task=:deploy)
  init_and_propagate(options, default_task.to_s) do |servers, config, shell|
    EY::Serverside::Deploy.new(servers, config, shell).send(default_task)
  end
end

#disable_maintenanceObject



54
55
56
57
58
# File 'lib/engineyard-serverside/cli.rb', line 54

def disable_maintenance
  init_and_propagate(options, 'disable_maintenance') do |servers, config, shell|
    EY::Serverside::Maintenance.new(servers, config, shell).manually_disable
  end
end

#enable_maintenanceObject



43
44
45
46
47
# File 'lib/engineyard-serverside/cli.rb', line 43

def enable_maintenance
  init_and_propagate(options, 'enable_maintenance') do |servers, config, shell|
    EY::Serverside::Maintenance.new(servers, config, shell).manually_enable
  end
end

#hook(hook_name) ⇒ Object



74
75
76
77
78
# File 'lib/engineyard-serverside/cli.rb', line 74

def hook(hook_name)
  init(options, "hook-#{hook_name}") do |config, shell|
    EY::Serverside::DeployHook.new(config, shell, hook_name).call
  end
end

#integrateObject



87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
# File 'lib/engineyard-serverside/cli.rb', line 87

def integrate
  app_dir = Pathname.new "/data/#{options[:app]}"
  current_app_dir = app_dir.join("current")

  # so that we deploy to the same place there that we have here
  integrate_options = options.dup
  integrate_options[:release_path] = current_app_dir.realpath.to_s

  # we have to deploy the same SHA there as here
  integrate_options[:branch] = current_app_dir.join('REVISION').read.strip

  init_and_propagate(integrate_options, 'integrate') do |servers, config, shell|

    # We have to rsync the entire app dir, so we need all the permissions to be correct!
    chown_command = "find #{app_dir} -not -user #{config.user} -or -not -group #{config.group} -exec chown #{config.user}:#{config.group} {} +"
    shell.logged_system "sudo sh -l -c '#{chown_command}'"

    servers.each do |server|
      shell.logged_system server.sync_directory_command(app_dir)
      # we're just about to recreate this, so it has to be gone
      # first. otherwise, non-idempotent deploy hooks could screw
      # things up, and since we don't control deploy hooks, we must
      # assume the worst.
      shell.logged_system server.command_on_server('sh -l -c', "rm -rf #{current_app_dir}")
    end

    # deploy local-ref to other instances into /data/$app/local-current
    EY::Serverside::Deploy.new(servers, config, shell).cached_deploy
  end
end

#restartObject



123
124
125
126
127
# File 'lib/engineyard-serverside/cli.rb', line 123

def restart
  init_and_propagate(options, 'restart') do |servers, config, shell|
    EY::Serverside::Deploy.new(servers, config, shell).restart_with_maintenance_page
  end
end